package com.demo;
import org.apache.commons.io.FileUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.mime.MultipartEntityBuilder;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.json.JSONArray;
import org.json.JSONObject;
import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.message.BasicNameValuePair;
 

public class AbbyyDemo {
	private static final String FILE_PATH = "D://Azure_templates//Type7.1_IndexTemp.pdf";
	private static final String UPLOAD_URL = "https://vantage-us.abbyy.com/api/publicapi/v1/transactions/launch?skillId=1bba94e0-5896-4034-bbab-e1f51f085515";
	private static final String TOKEN_URL = "https://vantage-us.abbyy.com/auth2/8b71b94c-4e08-414c-88f4-b319b64abc1b/connect/token";
	private static final String BASE_URL = "https://vantage-us.abbyy.com/api/publicapi/v1/transactions/";
	private static String token;
	private static String fileId;

	public static void main(String[] args) throws IOException, InterruptedException {
		System.out.println("Generating token...");
		token = generateToken();
		if (token == null) {
			System.err.println("Failed to generate token.");
			return;
		}
		System.out.println("");
		System.out.println("Token Generated Successfully.");
		System.out.println("");
//		System.out.println(token);
		
		System.out.println("Uploading file...");
		System.out.println("");
		String transactionId = uploadFile();
		if (transactionId == null) {
			System.err.println("Failed to upload file.");
			return;
		}
		
		System.out.println("");
		System.out.println("Checking status...");
		boolean isProcessingComplete = checkStatus(transactionId);
		if (!isProcessingComplete) {
			System.err.println("Processing not completed.");
			return;
		}
		System.out.println("");
		System.out.println("Downloading results...");
		System.out.println("");
		boolean isDownloadSuccessful = downloadResults(transactionId);
		if (!isDownloadSuccessful) {
			System.err.println("Failed to download results.");
		} else {
			System.out.println("Output JSON File Downloaded.");
			System.out.println("");
			System.out.println("Process completed successfully.");
		}
	}

	private static String generateToken() throws IOException {
		try (CloseableHttpClient client = HttpClients.createDefault()) {
			HttpPost request = new HttpPost(TOKEN_URL);
			request.setHeader("Content-Type", "application/x-www-form-urlencoded");
			// Added client credentials here
			request.setEntity((HttpEntity) new UrlEncodedFormEntity(
					Arrays.asList(new BasicNameValuePair("grant_type", "client_credentials"),
							new BasicNameValuePair("client_id", "gDNt3MeVG-vUMubC6tFsHkiRmpfSOe"),
							new BasicNameValuePair("client_secret", "Xd-lcTmjjm7DnfbnZMI42g0NCdwvHhd6xw"))));
			HttpResponse response = client.execute(request);
			if (response.getStatusLine().getStatusCode() == 200) {
				String responseString = EntityUtils.toString(response.getEntity());
				// Parsing of the token from the responseString
				System.out.println("Access Token Response: "+responseString);
				JSONObject jsonResponse = new JSONObject(responseString);
             return jsonResponse.getString("access_token");			

			} else {
				System.err.println(
						"Failed to generate token. HTTP Status Code: " + response.getStatusLine().getStatusCode());
				System.err.println("Response: " + EntityUtils.toString(response.getEntity()));
			}
		}
		return null;
	}

	private static String uploadFile() throws IOException {
		try (CloseableHttpClient client = HttpClients.createDefault()) {
			HttpPost request = new HttpPost(UPLOAD_URL);
			request.setHeader("Authorization", "Bearer " + token);
			File file = new File(FILE_PATH);
			if (!file.exists()) {
              System.err.println("File not found: " + FILE_PATH);
              return null;
            }
			
			MultipartEntityBuilder builder = MultipartEntityBuilder.create();
			builder.addBinaryBody("file", file, ContentType.APPLICATION_OCTET_STREAM, file.getName());
			request.setEntity(builder.build());
			HttpResponse response = client.execute(request);
			System.out.println(response);
			if (response.getStatusLine().getStatusCode() == 201) {
				String responseString = EntityUtils.toString(response.getEntity());
				System.out.println("");
              System.out.println("Upload successful. Response: " + responseString);
              System.out.println("");
              // To Extract transaction ID from the response
              JSONObject jsonResponse = new JSONObject(responseString);
              return jsonResponse.getString("transactionId");
			} else {
				System.err.println(
						"Failed to upload file. HTTP Status Code: " + response.getStatusLine().getStatusCode());
				System.err.println("Response: " + EntityUtils.toString(response.getEntity()));
			}
		}
		return null;
	}

	private static boolean checkStatus(String transactionId) throws IOException, InterruptedException {
		String statusUrl = BASE_URL + transactionId;
		try (CloseableHttpClient client = HttpClients.createDefault()) {
			HttpGet request = new HttpGet(statusUrl);
			request.setHeader("Authorization", "Bearer " + token);
			while (true) {
				HttpResponse response = client.execute(request);
				if (response.getStatusLine().getStatusCode() == 200) {
					String responseString = EntityUtils.toString(response.getEntity());
					// To check if processing is complete
					System.out.println("");
					System.out.println(responseString);
//					JSONObject jsonResponse = new JSONObject(responseString);
//		            return jsonResponse.getString("status");
					if (responseString.contains("Processed")) {
						// Parse JSON response
		                JSONObject jsonObject = new JSONObject(responseString);
		 
		                // Extracted documents array
		                JSONArray documentsArray = jsonObject.getJSONArray("documents");
		 
		                // Looping through each document and extract resultFiles
		                for (int i = 0; i < documentsArray.length(); i++) {
		                    JSONObject document = documentsArray.getJSONObject(i);
		                    JSONArray resultFilesArray = document.getJSONArray("resultFiles");
		 
		                    // To Extract and print fileIds from resultFiles
		                    for (int j = 0; j < resultFilesArray.length(); j++) {
		                        JSONObject resultFile = resultFilesArray.getJSONObject(j);
		                        fileId = resultFile.getString("fileId");
		                        System.out.println("");
		                        System.out.println("File Processed Successfully.");
		                        System.out.println("Result File ID: " + fileId);
		                        System.out.println("");
		                    }
		                }
						return true;
					} else {
						
						System.out.println("Processing... Waiting for 5 seconds.");
						System.out.println("");
						Thread.sleep(5000);
					}
				} else {
					System.err.println(
							"Failed to check status. HTTP Status Code: " + response.getStatusLine().getStatusCode());
					System.err.println("Response: " + EntityUtils.toString(response.getEntity()));
					return false;
				}
			}

		}
	}

	private static boolean downloadResults(String transactionId) throws IOException {
		String downloadUrl = BASE_URL + transactionId + "/files/" + fileId + "/download" ;
		try (CloseableHttpClient client = HttpClients.createDefault()) {
			HttpGet request = new HttpGet(downloadUrl);
			request.setHeader("Authorization", "Bearer " + token);
			HttpResponse response = client.execute(request);
			
			if (response.getStatusLine().getStatusCode() == 200) {
				// Saving the downloaded file
				File outputFile = new File("output.json");
				FileUtils.copyInputStreamToFile(response.getEntity().getContent(), outputFile);
				return true;
     		} else {
				System.err.println(
						"Failed to download results. HTTP Status Code: " + response.getStatusLine().getStatusCode());
				System.err.println("Response: " + EntityUtils.toString(response.getEntity()));
			}
		}
		return false;
	}
}